@@ -21,12 +21,13 @@ import aesara
 import numpy as np
 
 from aesara import function as aesara_function
-
-from pymc.aesaraf import inputvars, join_nonshared_inputs, make_shared_replacements
-from pymc_bart.bart import BARTRV
-from pymc_bart.tree import LeafNode, SplitNode, Tree
 from pymc.model import modelcontext
 from pymc.step_methods.arraystep import ArrayStepShared, Competence
+from pymc.aesaraf import inputvars, join_nonshared_inputs, make_shared_replacements
+
+from pymc_bart.bart import BARTRV
+from pymc_bart.tree import LeafNode, SplitNode, Tree
+
 
 _log = logging.getLogger("pymc")
 
@@ -40,7 +41,7 @@ class PGBART(ArrayStepShared):
     vars: list
         List of value variables for sampler
     num_particles : int
-        Number of particles for the conditional SMC sampler. Defaults to 40
+        Number of particles for the conditional SMC sampler. Defaults to 20
     batch : int or tuple
         Number of trees fitted per step. Defaults to  "auto", which is the 10% of the `m` trees
         during tuning and after tuning. If a tuple is passed the first element is the batch size
@@ -56,7 +57,7 @@ class PGBART(ArrayStepShared):
 
     def __init__(
         self,
-        vars=None,
+        vars=None,  # pylint: disable=redefined-builtin
         num_particles=20,
         batch="auto",
         model=None,
@@ -85,8 +86,8 @@ class PGBART(ArrayStepShared):
         self.alpha_vec = self.bart.split_prior
         self.init_mean = self.Y.mean()
         # if data is binary
-        Y_unique = np.unique(self.Y)
-        if Y_unique.size == 2 and np.all(Y_unique == [0, 1]):
+        y_unique = np.unique(self.Y)
+        if y_unique.size == 2 and np.all(y_unique == [0, 1]):
             mu_std = 3 / self.m**0.5
         # maybe we need to check for count data
         else:
@@ -99,10 +100,11 @@ class PGBART(ArrayStepShared):
         self.sum_trees = np.full((self.shape, self.Y.shape[0]), self.init_mean).astype(
             aesara.config.floatX
         )
-
-        self.a_tree = Tree.init_tree(
+        self.sum_trees_noi = self.sum_trees - (self.init_mean / self.m)
+        self.a_tree = Tree(
             leaf_node_value=self.init_mean / self.m,
             idx_data_points=np.arange(self.num_observations, dtype="int32"),
+            num_observations=self.num_observations,
             shape=self.shape,
         )
         self.normal = NormalSampler(mu_std, self.shape)
@@ -121,6 +123,7 @@ class PGBART(ArrayStepShared):
             else:
                 self.batch = (batch, batch)
 
+        self.num_particles = num_particles
         self.log_num_particles = np.log(num_particles)
         self.indices = list(range(2, num_particles))
         self.len_indices = len(self.indices)
@@ -183,14 +186,10 @@ class PGBART(ArrayStepShared):
 
             _, normalized_weights = self.normalize(particles)
             # Get the new tree and update
-            new_particle = np.random.choice(particles, p=normalized_weights)
-            new_tree = new_particle.tree
-
-            new_particle.log_weight = new_particle.old_likelihood_logp - self.log_num_particles
+            new_particle, new_tree = self.get_particle_tree(particles, normalized_weights)
             self.all_particles[tree_id] = new_particle
             self.sum_trees = self.sum_trees_noi + new_tree._predict()
             self.all_trees[tree_id] = new_tree.trim()
-
             used_variates = new_tree.get_split_variables()
 
             if self.tune:
@@ -213,10 +212,10 @@ class PGBART(ArrayStepShared):
         log_w = np.array([p.log_weight for p in particles])
         log_w_max = log_w.max()
         log_w_ = log_w - log_w_max
-        w_ = np.exp(log_w_)
-        w_sum = w_.sum()
+        wei = np.exp(log_w_)
+        w_sum = wei.sum()
         w_t = log_w_max + np.log(w_sum) - self.log_num_particles
-        normalized_weights = w_ / w_sum
+        normalized_weights = wei / w_sum
         # stabilize weights to avoid assigning exactly zero probability to a particle
         normalized_weights += 1e-12
 
@@ -228,7 +227,7 @@ class PGBART(ArrayStepShared):
 
         Ensure particles are copied only if needed.
         """
-        new_indices = systematic(normalized_weights)
+        new_indices = self.systematic(normalized_weights)
         seen = []
         new_particles = []
         for idx in new_indices:
@@ -242,6 +241,29 @@ class PGBART(ArrayStepShared):
 
         return particles
 
+    def get_particle_tree(self, particles, normalized_weights):
+        """
+        Sample a new particle, new tree and update log_weight
+        """
+        new_index = self.systematic(normalized_weights)[
+            discrete_uniform_sampler(self.num_particles)
+        ]
+        new_particle = particles[new_index - 2]
+        new_particle.log_weight = new_particle.old_likelihood_logp - self.log_num_particles
+        return new_particle, new_particle.tree
+
+    def systematic(self, normalized_weights):
+        """
+        Systematic resampling.
+
+        Return indices in the range 2, ..., len(normalized_weights)+2
+
+        Note: adapted from https://github.com/nchopin/particles
+        """
+        lnw = len(normalized_weights)
+        single_uniform = (self.uniform.random() + np.arange(lnw)) / lnw
+        return inverse_cdf(single_uniform, normalized_weights) + 2
+
     def init_particles(self, tree_id: int) -> np.ndarray:
         """Initialize particles."""
         p0 = self.all_particles[tree_id]
@@ -260,9 +282,9 @@ class PGBART(ArrayStepShared):
         for _ in self.indices:
             pt = ParticleTree(self.a_tree)
             if self.tune:
-                pt.kf = self.uniform.random()
+                pt.kfactor = self.uniform.random()
             else:
-                pt.kf = p0.kf
+                pt.kfactor = p0.kfactor
             particles.append(pt)
 
         return np.array(particles)
@@ -296,12 +318,14 @@ class PGBART(ArrayStepShared):
 class ParticleTree:
     """Particle tree."""
 
+    __slots__ = "tree", "expansion_nodes", "log_weight", "old_likelihood_logp", "kfactor"
+
     def __init__(self, tree):
         self.tree = tree.copy()  # keeps the tree that we care at the moment
         self.expansion_nodes = [0]
         self.log_weight = 0
         self.old_likelihood_logp = 0
-        self.kf = 0.75
+        self.kfactor = 0.75
 
     def sample_tree(
         self,
@@ -332,7 +356,7 @@ class ParticleTree:
                     sum_trees,
                     m,
                     normal,
-                    self.kf,
+                    self.kfactor,
                     shape,
                 )
                 if index_selected_predictor is not None:
@@ -352,7 +376,7 @@ class ParticleTree:
                     sum_trees[:, idx_data_points],
                     m,
                     normal,
-                    self.kf,
+                    self.kfactor,
                     shape,
                 )
                 leaf.value = node_value
@@ -369,10 +393,11 @@ class SampleSplittingVariable:
         self.enu = list(enumerate(np.cumsum(alpha_vec / alpha_vec.sum())))
 
     def rvs(self):
-        r = np.random.random()
-        for i, v in self.enu:
-            if r <= v:
+        rnd = np.random.random()
+        for i, val in self.enu:
+            if rnd <= val:
                 return i
+        return self.enu[-1]
 
 
 def compute_prior_probability(alpha):
@@ -412,7 +437,7 @@ def grow_tree(
     sum_trees,
     m,
     normal,
-    kf,
+    kfactor,
     shape,
 ):
     current_node = tree.get_node(index_leaf_node)
@@ -423,8 +448,9 @@ def grow_tree(
     available_splitting_values = X[idx_data_points, selected_predictor]
     split_value = get_split_value(available_splitting_values, idx_data_points, missing_data)
 
-    if split_value is not None:
-
+    if split_value is None:
+        index_selected_predictor = None
+    else:
         new_idx_data_points = get_new_idx_data_points(
             split_value, idx_data_points, selected_predictor, X
         )
@@ -440,7 +466,7 @@ def grow_tree(
                 sum_trees[:, idx_data_point],
                 m,
                 normal,
-                kf,
+                kfactor,
                 shape,
             )
 
@@ -463,7 +489,7 @@ def grow_tree(
         tree.set_node(new_nodes[0].index, new_nodes[0])
         tree.set_node(new_nodes[1].index, new_nodes[1])
 
-        return index_selected_predictor
+    return index_selected_predictor
 
 
 def get_new_idx_data_points(split_value, idx_data_points, selected_predictor, X):
@@ -483,44 +509,45 @@ def get_split_value(available_splitting_values, idx_data_points, missing_data):
             ~np.isnan(available_splitting_values)
         ]
 
+    split_value = None
     if available_splitting_values.size > 0:
         idx_selected_splitting_values = discrete_uniform_sampler(len(available_splitting_values))
         split_value = available_splitting_values[idx_selected_splitting_values]
 
-        return split_value
+    return split_value
 
 
-def draw_leaf_value(Y_mu_pred, m, normal, kf, shape):
+def draw_leaf_value(y_mu_pred, m, normal, kfactor, shape):
     """Draw Gaussian distributed leaf values."""
-    if Y_mu_pred.size == 0:
+    if y_mu_pred.size == 0:
         return np.zeros(shape)
     else:
-        norm = normal.random() * kf
-        if Y_mu_pred.size == 1:
-            mu_mean = np.full(shape, Y_mu_pred.item() / m)
+        norm = normal.random() * kfactor
+        if y_mu_pred.size == 1:
+            mu_mean = np.full(shape, y_mu_pred.item() / m)
         else:
-            mu_mean = fast_mean(Y_mu_pred) / m
+            mu_mean = fast_mean(y_mu_pred) / m
 
         draw = norm + mu_mean
         return draw
 
 
 @njit
-def fast_mean(a):
+def fast_mean(ari):
     """Use Numba to speed up the computation of the mean."""
 
-    if a.ndim == 1:
-        count = a.shape[0]
+    if ari.ndim == 1:
+        count = ari.shape[0]
         suma = 0
         for i in range(count):
-            suma += a[i]
+            suma += ari[i]
         return suma / count
-    elif a.ndim == 2:
-        res = np.zeros(a.shape[0])
-        count = a.shape[1]
-        for j in range(a.shape[0]):
+    else:
+        res = np.zeros(ari.shape[0])
+        count = ari.shape[1]
+        for j in range(ari.shape[0]):
             for i in range(count):
-                res[j] += a[j, i]
+                res[j] += ari[j, i]
         return res / count
 
 
@@ -577,19 +604,6 @@ class UniformSampler:
         )
 
 
-def systematic(normalized_weights):
-    """
-    Systematic resampling.
-
-    Return indices in the range 2, ..., len(normalized_weights)+2
-
-    Note: adapted from https://github.com/nchopin/particles
-    """
-    lnw = len(normalized_weights)
-    single_uniform = (np.random.rand(1) + np.arange(lnw)) / lnw
-    return inverse_cdf(single_uniform, normalized_weights) + 2
-
-
 @njit
 def inverse_cdf(single_uniform, normalized_weights):
     """
@@ -605,24 +619,24 @@ def inverse_cdf(single_uniform, normalized_weights):
 
     Returns
     -------
-    A: ndarray
+    new_indices: ndarray
         a vector of indices in range 2, ..., len(normalized_weights)+2
 
     Note: adapted from https://github.com/nchopin/particles
     """
-    j = 0
-    s = normalized_weights[0]
-    M = single_uniform.shape[0]
-    A = np.empty(M, dtype=np.int64)
-    for n in range(M):
-        while single_uniform[n] > s:
-            j += 1
-            s += normalized_weights[j]
-        A[n] = j
-    return A
+    idx = 0
+    a_weight = normalized_weights[0]
+    sul = len(single_uniform)
+    new_indices = np.empty(sul, dtype=np.int64)
+    for i in range(sul):
+        while single_uniform[i] > a_weight:
+            idx += 1
+            a_weight += normalized_weights[idx]
+        new_indices[i] = idx
+    return new_indices
 
 
-def logp(point, out_vars, vars, shared):
+def logp(point, out_vars, vars, shared):  # pylint: disable=redefined-builtin
     """Compile Aesara function of the model and the input and output variables.
 
     Parameters
@@ -635,6 +649,6 @@ def logp(point, out_vars, vars, shared):
         containing :class:`aesara.tensor.Tensor` for depended shared data
     """
     out_list, inarray0 = join_nonshared_inputs(point, out_vars, vars, shared)
-    f = aesara_function([inarray0], out_list[0])
-    f.trust_input = True
-    return f
+    function = aesara_function([inarray0], out_list[0])
+    function.trust_input = True
+    return function
